'''训练模型,将采集的图片数据集用神经网络训练出物体模型；
运行程序,待计算机自动训练,时间约1-5分钟不等,完成后,终端会显示“模型保存完成！”字样,并在当前文件目录下会自动生成“object_classification_model.h5”模型文件。'''
# 导入tensorflow深度学习框架
import tensorflow as tf
# 导入keras深度学习库
from tensorflow import keras
# 导入数据处理库
import numpy as np


# 数据预处理
# 将图片处理成预训练模型可以输入的格式
# 定义数据集路径
train_dir = 'dataset_object_classification'

# ImageDataGenerator为Keras的图像生成器，用于预处理图像数据
# mobilenet_v2.preprocess_input为mobilenet_v2的输入图像处理方法，用于将图像处理成适合mobilenet_v2模型的格式
datagen = keras.preprocessing.image.ImageDataGenerator(preprocessing_function=keras.applications.mobilenet_v2.preprocess_input)

# 图像生成器的flow_from_directory方法，负责生成批量数据，供模型训练
train_batches = datagen.flow_from_directory(        
        directory=train_dir,     #   directory：目标文件夹路径，在目标文件夹中，包含多个子文件夹，每个子文件夹都是一类，包含一类图像数据
        shuffle=True,            #     shuffle：是否打乱数据，设置为True，表示打乱每个子文件夹的数据处理顺序，否则按照子文件夹中图片名称的字母顺序处理数据
        target_size=(96,96),   # target_size：目标尺寸，所有图像将被重置为目标尺寸
        batch_size=10)           #  batch_size：数据批次大小，表示一批数据中包含的数据量，预处理时，会按批次处理数据

# train_batches返回的参数包括输入数据和标签数据

# 冻结预训练模型

# 基础模型用的是mobilenet_v2，输入图像尺寸为 224*224
base_model = keras.models.load_model("mobilenet_v2_96.h5",compile=False)  # 导入预训练模型
base_model.trainable = False                                 # 冻结预训练模型

# 这里的模型不含输出层，只有输入层和隐藏层
# 冻结后，权重等参数不再改变
# 冻结后的模型相当于只含输入层的特征提取器


# 创建神经网络

# 实例化一个神经网络模型
model = keras.Sequential(name="object_classification_model")

# 创建输入层
model.add(base_model)                                   # 定义输入层为预训练模型，输入尺寸即为 224*224
# 创建隐藏层
model.add(keras.layers.Dense(100, activation='relu'))   # 创建一层隐藏层，包括100个神经元
# 创建输出层
model.add(keras.layers.Dense(4, activation='softmax'))  # 定义输出层为3分类

# # 查看模型结构
# print("模型结构：")
# model.summary()


# 训练模型

# 设置参数
model.compile(
    optimizer=keras.optimizers.Adam(0.0001), 
    loss='categorical_crossentropy', 
    metrics=['accuracy']
)

# 训练模型
print("开始训练：")
model.fit(train_batches, epochs=5)  # 设置5轮训练
print("训练完成!")

# 保存模型
model.save("object_classification_model.h5")
print("模型保存完成！")
