'''在Pyboard屏幕上显示视频流画面并实时预测结果'''

# 导入tensorflow深度学习框架
import tensorflow as tf
# 导入keras深度学习库
from tensorflow import keras
# 导入OpenCV计算机视觉库
import cv2 
# 导入数据处理库
import numpy as np


# 导入物体分类模型
print("导入模型中...")
model_name = "object_classification_model.h5"
model = tf.keras.models.load_model(model_name)
print("{}模型导入完成".format(model_name))

# 图像预处理
def preprocess_img(frame):
    img = tf.image.resize(frame, (96, 96))                 # 重置尺寸
    img_array = keras.preprocessing.image.img_to_array(img)  # 将图像转为数组
    img_array = tf.expand_dims(img_array, 0)                 # 将图像数组增加一维，匹配模型输入
    img_array = img_array/255.                               # 归一化
    return img_array                                         # 返回预处理后的图像数据



# 绘制分类结果
def add_data(frame,predictions):
    res = frame                                        # 获取一帧图像
    box_startpoint = (30,10)                           # 定义坐标
    class_names = ["01Apple","02Banana","03Watermelon","04Others"]    # 定义分类结果
    
    # # 在屏幕左上方，绘制各个分类结果的置信度直方图
    # for idx,pred in enumerate(predictions.numpy()[0]):
        # # 绘制文字
        # res = cv2.putText(res,  
        #     str(class_names[idx]), 
        #     (box_startpoint[0]-20, box_startpoint[1]+idx*40+10), #+20
        #     cv2.FONT_HERSHEY_SIMPLEX, 0.5, 
        #     (0, 0, 255), 
        #     2, 
        #     cv2.LINE_4)
        # # 绘制直方图
        # res = cv2.rectangle(res,
        #     (box_startpoint[0]+70,box_startpoint[1]+idx*40),
        #     (int(box_startpoint[0]+100*np.around(predictions.numpy(),2)[0][idx])+70,box_startpoint[1]+idx*40+15),
        #     (0,0,255),
        #     -1)
    # 绘制直方图
    res = cv2.rectangle(res,
        (0,0),
        (160,35),
        (255,255,255),
        -1)    
    # 在屏幕下方，绘制预测结果"predicted result :"+
    res = cv2.putText(res,str(class_names[predictions.numpy().argmax()]), 
        (0,25), # coor
        cv2.FONT_HERSHEY_SIMPLEX, 1, 
        (255, 0, 0), 
        2, 
        cv2.LINE_4)


    res = str(class_names[predictions.numpy().argmax()])
    return res
    
# 初始化屏幕
window_name = 'frame'
cv2.namedWindow(window_name, cv2.WND_PROP_FULLSCREEN)
cv2.setWindowProperty(window_name, cv2.WND_PROP_FULLSCREEN, cv2.WINDOW_FULLSCREEN)
# whole_frame = np.zeros((480,800,3), dtype="uint8")

# 初始化摄像头
while True: 
    cap = cv2.VideoCapture(0)            # 获取摄像头图像
    cap.set(cv2.CAP_PROP_FRAME_WIDTH, 240)
    cap.set(cv2.CAP_PROP_FRAME_HEIGHT, 320)
    cap.set(cv2.CAP_PROP_BUFFERSIZE, 1)  # 设置内部缓冲存储器中的帧数量
    if cap is not None:                  # 如果没有连接摄像头，就等待
        break

while True:
    # 逐帧捕获摄像头的图像
    ret, frame = cap.read() # ret为True或False，表示有没有读到图像，frame表示当前截取一帧的图像
    img_array = preprocess_img(frame)         # 对一帧图像做预处理
    predictions = model.predict(img_array)    # 模型预测（整体概率）
    predictions = tf.nn.softmax(predictions)  # 输出预测结果
    new_frame = add_data(frame,predictions)   # 对每一帧图像绘制分类结果
    # frame=cv2ImgAddText(frame,new_frame, (10, 30),(0, 255, 0), 30)
    print(new_frame)

    # 显示图像    
    cv2.imshow('frame',frame)
    
    # 按下熊猫键盘“A”键，退出程序
    if cv2.waitKey(1) & 0xFF == ord('b'):
        break

#清空显示
cap.release()
cv2.destroyAllWindows()
